import 'package:ml_dataframe/ml_dataframe.dart';
import 'package:ml_dataframe/src/data_frame/data_frame_helpers.dart';
import 'package:ml_dataframe/src/data_frame/data_frame_impl.dart';
import 'package:ml_dataframe/src/data_selector/data_selector.dart';
import 'package:ml_dataframe/src/numerical_converter/numerical_converter_impl.dart';
import 'package:ml_linalg/dtype.dart';
import 'package:quiver/iterables.dart';

DataFrame fromRawData(Iterable<Iterable<dynamic>> data, {
  bool headerExists = true,
  Iterable<String> predefinedHeader = const [],
  String autoHeaderPrefix = defaultHeaderPrefix,
  Iterable<int> columns = const [],
  Iterable<String> columnNames = const [],
  DType dtype = DType.float32,
}) {
  final columnsNum = columns.isNotEmpty
      ? columns.length
      : data.first.length;

  final header = getHeader(
      columnsNum,
      autoHeaderPrefix,
      headerExists
          ? data.first.map((dynamic el) => el.toString())
          : [],
      predefinedHeader);

  final defaultIndices = count(0).take(columnsNum);

  final filteredIndices = enumerate(header)
      .where((indexedName) => columnNames.contains(indexedName.value))
      .map((indexedName) => indexedName.index);

  final columnIndices = columns.isNotEmpty
      ? columns
      : predefinedHeader.isNotEmpty || columnNames.isEmpty
          ? defaultIndices
          : filteredIndices;

  final originalHeadlessData = headerExists
      ? data.skip(1)
      : data;

  final selectedData = DataSelector(columnIndices)
      .select(originalHeadlessData);

  final selectedHeader = (predefinedHeader.isNotEmpty
      ? enumerate(header)
      : enumerate(header)
          .where((indexedName) => columnIndices.isNotEmpty
            ? columnIndices.contains(indexedName.index)
            : true)
      )
      .map((indexedName) => indexedName.value);

  return DataFrameImpl(selectedData, selectedHeader,
      NumericalConverterImpl(false), dtype);
}
